from unittest import TestCase
from canari.maltego.transform import Transform
from canari.framework import *

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


class TransformTests(TestCase):
    """
    TODO: Test cases for ExternalCommand decorator.
    """

    def get_transform_name(self, t):
        return '.'.join([t.__module__.split('.', 1)[0], t.__name__])

    def test_create_default_transform(self):
        class MyTransform(Transform):
            """This is a test."""
            pass
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, 'This is a test.')
        self.assertEqual(transform.help_url, '')
        self.assertFalse(transform.debug)
        self.assertFalse(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertFalse(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_default_superuser_transform(self):
        @RequireSuperUser
        class MyTransform(Transform):
            """This is a test."""
            pass
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, 'This is a test.')
        self.assertEqual(transform.help_url, '')
        self.assertFalse(transform.debug)
        self.assertTrue(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertFalse(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_default_superuser_debug_transform(self):
        @RequireSuperUser
        @EnableDebugWindow
        class MyTransform(Transform):
            """This is a test."""
            pass
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, 'This is a test.')
        self.assertEqual(transform.help_url, '')
        self.assertTrue(transform.debug)
        self.assertTrue(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertFalse(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_default_superuser_debug_remote_transform(self):
        @RequireSuperUser
        @EnableDebugWindow
        @EnableRemoteExecution
        class MyTransform(Transform):
            """This is a test."""
            pass
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, 'This is a test.')
        self.assertEqual(transform.help_url, '')
        self.assertTrue(transform.debug)
        self.assertTrue(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertTrue(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_default_superuser_debug_remote_deprecated_transform(self):
        @RequireSuperUser
        @EnableDebugWindow
        @EnableRemoteExecution
        @Deprecated
        class MyTransform(Transform):
            """This is a test."""
            pass
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, 'This is a test.')
        self.assertEqual(transform.help_url, '')
        self.assertTrue(transform.debug)
        self.assertTrue(transform.superuser)
        self.assertTrue(transform.deprecated)
        self.assertTrue(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_no_description_transform(self):
        class MyTransform(Transform):
            pass
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, '')
        self.assertEqual(transform.help_url, '')
        self.assertFalse(transform.debug)
        self.assertFalse(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertFalse(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_explicit_name_transform(self):
        class MyTransform(Transform):
            name = 'foobar.Transform'
        transform = MyTransform()
        self.assertEqual(transform.name, 'foobar.Transform')
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, '')
        self.assertEqual(transform.help_url, '')
        self.assertFalse(transform.debug)
        self.assertFalse(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertFalse(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_explicit_display_name_transform(self):
        class MyTransform(Transform):
            display_name = 'Foo'
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'Foo')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, '')
        self.assertEqual(transform.help_url, '')
        self.assertFalse(transform.debug)
        self.assertFalse(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertFalse(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_explicit_description_transform(self):
        class MyTransform(Transform):
            description = 'foo'
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, 'foo')
        self.assertEqual(transform.help_url, '')
        self.assertFalse(transform.debug)
        self.assertFalse(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertFalse(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_explicit_help_url_transform(self):
        class MyTransform(Transform):
            help_url = 'foo'
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, '')
        self.assertEqual(transform.help_url, 'foo')
        self.assertFalse(transform.debug)
        self.assertFalse(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertFalse(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_explicit_author_transform(self):
        class MyTransform(Transform):
            author = 'foo'
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, 'foo')
        self.assertEqual(transform.description, '')
        self.assertEqual(transform.help_url, '')
        self.assertFalse(transform.debug)
        self.assertFalse(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertFalse(transform.remote)
        self.assertEqual(transform.transform_set, self.__module__.split('.', 1)[0].title())

    def test_create_explicit_transform_set(self):
        class MyTransform(Transform):
            transform_set = 'foo'
        transform = MyTransform()
        self.assertEqual(transform.name, self.get_transform_name(MyTransform))
        self.assertEqual(transform.display_name, 'My Transform')
        self.assertEqual(transform.author, '')
        self.assertEqual(transform.description, '')
        self.assertEqual(transform.help_url, '')
        self.assertFalse(transform.debug)
        self.assertFalse(transform.superuser)
        self.assertFalse(transform.deprecated)
        self.assertFalse(transform.remote)
        self.assertEqual(transform.transform_set, 'foo')


from unittest import TestCase
from canari.mode import *
import sys

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


class ModeTests(TestCase):
    
    def setUp(self):
        self.tests = dict(
            is_remote_exec_mode=is_remote_exec_mode,
            is_plume_exec_mode=is_plume_exec_mode,
            is_local_exec_mode=is_local_exec_mode,
            is_debug_exec_mode=is_debug_exec_mode,
            is_dispatch_exec_mode=is_dispatch_exec_mode,
            is_unknown_exec_mode=is_unknown_exec_mode,
            is_local_debug_exec_mode=is_local_debug_exec_mode,
            is_local_dispatch_exec_mode=is_local_dispatch_exec_mode,
            is_local_unknown_exec_mode=is_local_unknown_exec_mode,
            is_remote_plume_debug_exec_mode=is_remote_plume_debug_exec_mode,
            is_remote_plume_dispatch_exec_mode=is_remote_plume_dispatch_exec_mode,
            is_remote_unknown_exec_mode=is_remote_unknown_exec_mode,
        )

    def test_initial_load(self):
        self.assertEqual(CanariMode.Unknown, get_canari_mode())
        
    def set_canari_mode(self, mode, truth_table):
        set_canari_mode(mode)
        self.assertEqual(mode, get_canari_mode())
        for name, func in self.tests.iteritems():
            expected_result = truth_table.issuperset([name])
            self.assertEqual(
                expected_result, bool(func()),
                '%s returned %s instead of %s (expected).' % (name, not expected_result, expected_result)
            )

    def test_set_remote_exec_mode(self):
        self.set_canari_mode(
            CanariMode.Remote,
            {'is_remote_exec_mode'}
        )

    def test_set_plume_exec_mode(self):
        self.set_canari_mode(
            CanariMode.Plume,
            {'is_plume_exec_mode'}
        )

    def test_set_local_exec_mode(self):
        self.set_canari_mode(
            CanariMode.Local,
            {'is_local_exec_mode'}
        )

    def test_set_debug_exec_mode(self):
        self.set_canari_mode(
            CanariMode.Debug,
            {'is_debug_exec_mode'}
        )

    def test_set_dispatch_exec_mode(self):
        self.set_canari_mode(
            CanariMode.Dispatch,
            {'is_dispatch_exec_mode'}
        )

    def test_set_unknown_exec_mode(self):
        self.set_canari_mode(
            CanariMode.Unknown,
            {'is_unknown_exec_mode'}
        )

    def test_set_local_debug_exec_mode(self):
        self.set_canari_mode(
            CanariMode.LocalDebug,
            {'is_debug_exec_mode', 'is_local_exec_mode', 'is_local_debug_exec_mode'}
        )

    def test_set_local_dispatch_exec_mode(self):
        self.set_canari_mode(
            CanariMode.LocalDispatch,
            {'is_dispatch_exec_mode', 'is_local_exec_mode', 'is_local_dispatch_exec_mode'}
        )

    def test_set_local_unknown_exec_mode(self):
        self.set_canari_mode(
            CanariMode.LocalUnknown,
            {'is_unknown_exec_mode', 'is_local_exec_mode', 'is_local_unknown_exec_mode'}
        )

    def test_set_remote_plume_debug_exec_mode(self):
        self.set_canari_mode(
            CanariMode.RemotePlumeDebug,
            {'is_plume_exec_mode', 'is_remote_exec_mode',
             'is_debug_exec_mode', 'is_remote_plume_debug_exec_mode'}
        )

    def test_set_remote_plume_dispatch_exec_mode(self):
        self.set_canari_mode(
            CanariMode.RemotePlumeDispatch,
            {'is_plume_exec_mode', 'is_remote_exec_mode',
             'is_dispatch_exec_mode', 'is_remote_plume_dispatch_exec_mode'}
        )

    def test_set_remote_unknown_exec_mode(self):
        self.set_canari_mode(
            CanariMode.RemoteUnknown,
            {'is_remote_exec_mode', 'is_unknown_exec_mode', 'is_remote_unknown_exec_mode'}
        )

    def tearDown(self):
        sys.modules.pop('canari.config', None)


import unittest

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

if __name__ == "__main__":
    all_tests = unittest.TestLoader().discover('canari.unittests', pattern='*.py')
    unittest.TextTestRunner().run(all_tests)



from unittest import TestCase
from canari.framework import *
from canari.maltego.entities import *
from canari.maltego.transform import Transform

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


class FrameworkTests(TestCase):

    def create_and_test_transform_function(self, is_superuser=False, is_deprecated=False, is_debug=False,
                                           is_remote=False, is_external=False, **kwargs):
        class TestTransform(Transform):
            pass

        for k, v in kwargs.iteritems():
            setattr(TestTransform, k, v)

        if is_superuser:
            RequireSuperUser(TestTransform)
        if is_deprecated:
            Deprecated(TestTransform)
        if is_debug:
            EnableDebugWindow(TestTransform)
        if is_remote:
            EnableRemoteExecution(TestTransform)
        # if is_external:
        #     ExternalCommand(TestTransform)

        return self.assertConfigure(TestTransform(), is_superuser, is_deprecated, is_debug, is_remote,
                                    is_external, **kwargs)

    def assertConfigure(self, transform, is_superuser=False, is_deprecated=False, is_debug=False, is_remote=False,
                        is_external=False, **kwargs):
        self.assertEqual(transform.superuser, is_superuser)
        self.assertEqual(transform.deprecated, is_deprecated)
        self.assertEqual(transform.debug, is_debug)
        self.assertEqual(transform.remote, is_remote)

    def test_deprecated_transform(self):
        self.create_and_test_transform_function(is_deprecated=True)

    def test_superuser_transform(self):
        self.create_and_test_transform_function(is_superuser=True)

    def test_debug_transform(self):
        self.create_and_test_transform_function(is_debug=True)

    def test_remote_transform(self):
        self.create_and_test_transform_function(is_remote=True)


#!/usr/bin/env python

# Builtin imports
import logging
import os
import sys
import tempfile
import traceback
from ConfigParser import NoSectionError
from hashlib import md5
from logging.handlers import RotatingFileHandler

from flask import Flask, Response, request, safe_join, url_for

import canari.resource
from canari.commands.common import fix_binpath, fix_pypath
from canari.mode import set_canari_mode, CanariMode
from canari.config import load_config, OPTION_REMOTE_PATH
from canari.maltego.entities import Phrase, Unknown
from canari.maltego.message import (MaltegoMessage, MaltegoTransformResponseMessage, MaltegoTransformExceptionMessage,
                                    MaltegoException)
from canari.maltego.transform import Transform
from canari.pkgutils.transform import TransformDistribution

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.5'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'application'
]

# Initialize Canari modes and bin path
set_canari_mode(CanariMode.RemotePlumeDispatch)
fix_binpath(load_config()[OPTION_REMOTE_PATH])
fix_pypath()


def get_image_path(i):
    return os.path.join('static', md5(i).hexdigest())


def get_image_url(i):
    return '%s/static/%s' % (request.host_url, md5(i).hexdigest())


# Monkey patch our resource lib to automatically rewrite icon urls
_icon_resource = canari.resource.icon_resource
canari.resource.icon_resource = lambda name, pkg=None: get_image_url(_icon_resource(name, pkg))

_calling_package = canari.resource.calling_package
canari.resource.calling_package = lambda frame=4: _calling_package(frame)

# Use temporary directory as Python Egg Cache
os.environ['PYTHON_EGG_CACHE'] = tempfile.gettempdir()


class Version(Transform):
    input_type = Phrase

    def do_transform(self, r, res, config):
        if r.entity.value == 'version':
            res += Phrase('Canari v%s' % __version__)
        return res


class Plume(Flask):
    four_o_four = 'Whaaaaat?'

    def __init__(self, import_name, *args, **kwargs):
        super(Plume, self).__init__(import_name, *args, **kwargs)
        self.transforms = {}
        self.resources = []
        self._initialize()

    def _copy_images(self, pkg):
        if pkg.endswith('.transforms'):
            pkg = pkg.replace('.transforms', '')
        for i in canari.resource.image_resources(pkg):
            img_name = get_image_path(i)
            self.resources.append(img_name)
            if not os.path.exists(img_name):
                print 'Copying %s to %s...' % (i, img_name)
                with open(i, mode='rb') as src:
                    with open(img_name, mode="wb") as dst:
                        dst.write(src.read())

    def _initialize(self):

        packages = None

        # Read packages that are to be loaded at runtime
        try:
            config = load_config()
            packages = config['canari.remote.packages']
        except NoSectionError:
            sys.stderr.write('Exiting... You did not specify a [canari.remote] section and a "packages" '
                             'option in your canari.conf file!\n')
            exit(-1)

        # Is packages not blank
        if not packages:
            sys.stderr.write(
                    'Exiting... You did not specify any transform packages to load in your canari.conf file!\n')
            exit(-1)
        elif isinstance(packages, basestring):
            packages = [packages]

        # Create the static directory for static file loading
        if not os.path.exists('static'):
            os.mkdir('static', 0755)

        # Iterate through the list of packages to load
        for p in packages:
            # Copy all the image resource files in case they are used as entity icons

            distribution = TransformDistribution(p)

            sys.stderr.write('Loading transform package %s\n' % repr(p))

            for transform in distribution.remote_transforms:
                transform_name = transform().name
                sys.stderr.write('Loading transform %s at /%s...\n' % (repr(transform_name), transform_name))
                if os.name == 'posix' and transform.superuser and os.geteuid() and __name__.startswith('_mod_wsgi_'):
                    sys.stderr.write('WARNING: mod_wsgi does not allow applications to run with root privileges. '
                                     'Transform %s ignored...\n' % repr(transform_name))
                    continue

                self.transforms[transform_name] = transform

            self._copy_images(p)

        if not self.transforms:
            sys.stderr.write('Exiting... Your transform packages have no remote transforms.\n')
            exit(-1)

        self.transforms['canari.Version'] = Version


# Create our Flask app.
application = Plume(__name__)


def croak(cause):
    """Throw an exception in the Maltego GUI containing cause.

    :param cause: a string containing the issue description.
    """
    return MaltegoMessage(
            message=MaltegoTransformExceptionMessage(
                    exceptions=[
                        MaltegoException(cause)
                    ]
            )
    ).render()


def message(msg):
    """Write a MaltegoMessage to stdout and exit successfully"""
    v = MaltegoMessage(message=msg).render()
    return Response(v, status=200, mimetype='text/xml')


def do_transform(transform):
    try:
        # Let's get an XML object tree
        req = MaltegoMessage.parse(request.data).message

        # If our transform define an input entity type then we should check
        # whether the request contains the right type
        if transform.input_type and transform.input_type is not Unknown and \
                not isinstance(req.entity, transform.input_type):
            return Response(application.four_o_four, status=404)

        # Execute it!
        msg = transform().do_transform(
                req,
                MaltegoTransformResponseMessage(),
                load_config()
        )

        # Let's serialize the return response and clean up whatever mess was left behind
        if isinstance(msg, MaltegoTransformResponseMessage):
            return message(msg)
        else:
            raise MaltegoException(str(msg))

    # Unless we croaked somewhere, then we need to fix things up here...
    except MaltegoException, me:
        return croak(str(me))
    except Exception:
        if application.debug:
            return croak(traceback.format_exc())
        else:
            return croak('Transform execution failed.')


# This is where the TDS will ask: "Are you a transform?" and we say "200 - Yes I am!" or "404 - PFO"
@application.route('/<transform_name>', methods=['GET'])
def transform_checker(transform_name):
    if transform_name in application.transforms:
        return Response('Yes?', status=200)
    return Response(application.four_o_four, status=404)


@application.route('/static/<resource_name>', methods=['GET'])
def static_fetcher(resource_name):
    resource_name = safe_join('static', resource_name)
    if resource_name in application.resources:
        return Response(file(resource_name, mode='rb').read(), status=200, mimetype='application/octet-stream')
    return Response(application.four_o_four, status=404)


# This is where we process a transform request.
@application.route('/<transform_name>', methods=['POST'])
def transform_runner(transform_name):
    if transform_name not in application.transforms:
        return Response(application.four_o_four, status=404)
    return do_transform(application.transforms[transform_name])


# To run Flask standalone just type `python -m canari.tas.plume`
if __name__ == '__main__':
    set_canari_mode(CanariMode.RemotePlumeDebug)
    handler = RotatingFileHandler('plume.log', maxBytes=10000, backupCount=1)
    handler.setLevel(logging.INFO)
    application.logger.addHandler(handler)
    application.run(debug=True)



import os
import re
import string
from ConfigParser import SafeConfigParser, NoOptionError, NoSectionError
from urlparse import urlparse

from canari.mode import is_local_exec_mode, is_remote_exec_mode
from canari.utils.fs import PushDir
from resource import global_config
from utils.wordlist import wordlist

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'CanariConfigParser',
    'load_config',
    'NoOptionError',
    'NoSectionError'
]

SECTION_LOCAL = 'canari.local'

SECTION_REMOTE = 'canari.remote'

OPTION_LOCAL_CONFIGS = 'canari.local.configs'
OPTION_REMOTE_CONFIGS = 'canari.remote.configs'

OPTION_LOCAL_PATH = 'canari.local.path'
OPTION_REMOTE_PATH = 'canari.remote.path'

OPTION_REMOTE_PACKAGES = 'canari.remote.packages'


class CanariConfigParser(SafeConfigParser):

    @staticmethod
    def _interpolate_environment_variables(value):
        if isinstance(value, str):
            value = string.Template(value).safe_substitute(os.environ)
        return value

    def _interpolate(self, section, option, value, d):
        return SafeConfigParser._interpolate(self, section, option, self._interpolate_environment_variables(value), d)

    def __iadd__(self, other):
        self.add_section(other)
        return self

    def __isub__(self, other):
        self.remove_section(other)
        return self

    def _parse_value(self, value):
        if value.startswith('object://') and is_local_exec_mode():
            r = urlparse(value)
            try:
                v = r.path.lstrip('/')
                m = __import__(r.netloc, globals(), locals(), [v])
                value = m.__dict__[v]
            except ImportError:
                pass
        elif re.match(r'^\d+$', value):
            value = int(value)
        elif re.match(r'^\d+\.\d+$', value):
            value = float(value)
        elif re.search(r'\s*(?<=[^\\]),+\s*', value):
            l = re.split(r'\s*(?<=[^\\]),+\s*', value)
            value = []
            for v in l:
                value.append(self._parse_value(v))
        else:
            value = value.replace(r'\,', ',')
        return value

    def _render_value(self, value):
        if isinstance(value, str):
            if value.strip().startswith('object://'):
                raise ValueError('Cannot set object option.')
            value = value.replace(',', '\\,')
        elif isinstance(value, list):
            value = ','.join([self._render_value(v) for v in value])
        elif callable(value):
            raise ValueError('Cannot set object option.')
        else:
            value = str(value)
        return value

    def _get_option_value(self, key, raise_on_empty_option=False):
        if not key and raise_on_empty_option:
            raise KeyError("Invalid key %r, must be in '<section>.<option>'" % key)
        if self.has_section(key):
            return key, ''
        return key.rsplit('.', 1) if '.' in key else (key, '')

    def __getitem__(self, key):
        section, option = self._get_option_value(key, True)
        value = self.get(section, option)
        value = self._parse_value(value)
        if option == 'wordlist':
            value = wordlist(value)
        return value

    def __setitem__(self, key, value):
        section, option = self._get_option_value(key, True)
        if not self.has_section(section):
            self.add_section(section)
        self.set(section, option, self._render_value(value))

    def __delitem__(self, key):
        section, option = self._get_option_value(key)
        if option:
            self.remove_option(section, option)
        else:
            self.remove_section(section)

    def __contains__(self, key):
        section, option = self._get_option_value(key)
        if option:
            return self.has_option(section, option)
        return self.has_section(section)

    def get_as_list(self, key):
        opt = self.__getitem__(key)
        if isinstance(opt, basestring):
            opt = [opt] if opt else []
        return opt

    def update(self, other):
        if not isinstance(other, CanariConfigParser):
            raise ValueError('Expected a CanariConfigParser, got %r instead' % type(other).__name__)
        self._sections.update(other._sections)


def load_config(config_file=None, recursive_load=True):
    if not config_file:
        config_file = os.path.join(os.getcwd(), 'canari.conf')
        if not os.path.lexists(config_file):
            config_file = os.path.join(os.path.expanduser('~'), '.canari', 'canari.conf')

    with PushDir(os.path.dirname(config_file)):
        config_parser = CanariConfigParser()
        config_parser.read([global_config, config_file])
        if recursive_load:
            if is_remote_exec_mode() and OPTION_REMOTE_CONFIGS in config_parser:
                config_parser.read(config_parser[OPTION_REMOTE_CONFIGS])
            elif OPTION_LOCAL_CONFIGS in config_parser:
                config_parser.read(config_parser[OPTION_LOCAL_CONFIGS])

        return config_parser



from datetime import date, datetime, timedelta
from canari.maltego.message import Entity, Field, StringEntityField, IntegerEntityField, FloatEntityField, \
    BooleanEntityField, EnumEntityField, LongEntityField, DateTimeEntityField, DateEntityField, TimeSpan, \
    TimeSpanEntityField, RegexEntityField, ColorEntityField, ValidationError
from unittest import TestCase

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'EntityTests'
]


class EntityTests(TestCase):

    def test_basic_uninitialized_entity(self):
        class TestEntity(Entity):
            pass
        self.assertEqual(TestEntity._alias_, TestEntity.__name__)
        self.assertEqual(TestEntity._namespace_, TestEntity.__module__.split('.', 1)[0])
        self.assertEqual(TestEntity._type_, '%s.%s' % (TestEntity._namespace_, TestEntity.__name__))

    def test_basic_uninitialized_entity_with_explicit_namespace(self):
        class TestEntity(Entity):
            _namespace_ = 'foo'
        self.assertEqual(TestEntity._alias_, TestEntity.__name__)
        self.assertEqual(TestEntity._namespace_, 'foo')
        self.assertEqual(TestEntity._type_, '%s.%s' % (TestEntity._namespace_, TestEntity.__name__))

    def test_basic_uninitialized_entity_with_explicit_type(self):
        class TestEntity(Entity):
            _type_ = 'foo'
        self.assertEqual(TestEntity._alias_, TestEntity.__name__)
        self.assertEqual(TestEntity._namespace_, TestEntity.__module__.split('.', 1)[0])
        self.assertEqual(TestEntity._type_, 'foo')

    def test_basic_uninitialized_entity_with_explicit_alias(self):
        class TestEntity(Entity):
            _alias_ = 'foo'
        self.assertEqual(TestEntity._alias_, 'foo')
        self.assertEqual(TestEntity._namespace_, TestEntity.__module__.split('.', 1)[0])
        self.assertEqual(TestEntity._type_, '%s.%s' % (TestEntity._namespace_, TestEntity.__name__))

    def test_basic_entity_with_explicit_string_property_and_value(self):
        class TestEntity(Entity):
            foo = StringEntityField('foo.bar')
        t = TestEntity('value', foo='random value')
        self.assertEqual(t.value, 'value')
        self.assertEqual(t.foo, 'random value')
        t.foo = 1
        self.assertEqual(t.foo, 1)

    def test_basic_entity_with_dynamic_field_and_explicity_field(self):
        class TestEntity(Entity):
            foo = StringEntityField('foo.bar')
        t = TestEntity('value', foo='random value')
        t += Field('foo.bar', 1)
        self.assertEqual(t.foo, 1)

    def test_basic_entity_with_all_static_property_types(self):
        class TestEntity(Entity):
            str = StringEntityField('type.str')
            int = IntegerEntityField('type.int')
            float = FloatEntityField('type.float')
            bool = BooleanEntityField('type.bool')
            enum = EnumEntityField('type.enum', choices=[2, 1, 0])
            date = DateEntityField('type.date')
            datetime = DateTimeEntityField('type.datetime')
            timespan = TimeSpanEntityField('type.timespan')
            color = ColorEntityField('type.color')

        t = TestEntity(
            'value',
            str='str',
            int=1,
            float=1.0,
            bool=True,
            enum=2,
            date=date(1900, 01, 01),
            datetime=datetime(1900, 01, 01),
            timespan=timedelta(days=2, minutes=1, seconds=60),
            color='#ffffff'
        )

        self.assertEqual(t.value, 'value')
        self.assertEqual(t.str, 'str')
        self.assertEqual(t.int, 1)
        self.assertEqual(t.float, 1.0)
        self.assertEqual(t.bool, True)
        self.assertEqual(t.enum, '2')
        self.assertEqual(t.date, date(1900, 01, 01))
        self.assertEqual(t.datetime, datetime(1900, 01, 01))
        self.assertEqual(t.timespan, timedelta(days=2, minutes=1, seconds=60))
        self.assertEqual(t.color, '#ffffff')

        def assign_bad_integer():
            t.int = 'str'
        self.assertRaises(ValidationError, assign_bad_integer)
        self.assertEqual(t.int, 1)

        def assign_bad_float():
            t.float = 'str'
        self.assertRaises(ValidationError, assign_bad_float)
        self.assertEqual(t.float, 1.0)

        def assign_bad_bool():
            t.bool = 'str'
        self.assertRaises(ValidationError, assign_bad_bool)
        self.assertEqual(t.bool, True)

        def assign_bad_enum():
            t.enum = 3
        self.assertRaises(ValidationError, assign_bad_enum)
        self.assertEqual(t.enum, '2')

        def assign_bad_date():
            t.date = '24-01-01'
        self.assertRaises(ValidationError, assign_bad_date)
        self.assertEqual(t.date, date(1900, 01, 01))

        def assign_bad_datetime():
            t.date = '24-01-01'
        self.assertRaises(ValidationError, assign_bad_datetime)
        self.assertEqual(t.datetime, datetime(1900, 01, 01))

        def assign_bad_timespan():
            t.date = '24-01-01'
        self.assertRaises(ValidationError, assign_bad_timespan)
        self.assertEqual(t.timespan, timedelta(days=2, minutes=1, seconds=60))

        def assign_bad_color():
            t.color = '24-01-01'
        self.assertRaises(ValidationError, assign_bad_color)
        self.assertEqual(t.color, '#ffffff')

    def test_property_type_retrieval(self):
        class TestEntity(Entity):
            str = StringEntityField('type.str')
            int = IntegerEntityField('type.int')
            float = FloatEntityField('type.float')
            bool = BooleanEntityField('type.bool')
            enum = EnumEntityField('type.enum', choices=[2, 1, 0])
            date = DateEntityField('type.date')
            datetime = DateTimeEntityField('type.datetime')
            timespan = TimeSpanEntityField('type.timespan')
            color = ColorEntityField('type.color')

        self.assertEqual(TestEntity.str.name, 'type.str')
        self.assertEqual(TestEntity.int.name, 'type.int')
        self.assertEqual(TestEntity.float.name, 'type.float')
        self.assertEqual(TestEntity.bool.name, 'type.bool')
        self.assertEqual(TestEntity.enum.name, 'type.enum')
        self.assertEqual(TestEntity.date.name, 'type.date')
        self.assertEqual(TestEntity.datetime.name, 'type.datetime')
        self.assertEqual(TestEntity.timespan.name, 'type.timespan')
        self.assertEqual(TestEntity.color.name, 'type.color')


from unittest import TestCase
from canari.resource import *
import os

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


class ResourceTests(TestCase):

    def test_external_resource(self):
        resource = external_resource('wordlist.txt', 'canari.unittests.resources')
        self.assertTrue(resource.endswith(os.path.join('canari', 'unittests', 'resources', 'wordlist.txt')))
        self.assertTrue(os.path.lexists(resource))

    def test_icon_resource(self):
        icon = icon_resource('0.png', 'canari.unittests.resources.images')
        self.assertTrue(icon.startswith('file:///') and icon.endswith('canari/unittests/resources/images/0.png'))
        self.assertTrue(os.path.lexists(icon.replace('file://', '')))

    def test_image_resources(self):
        images = image_resources('canari.unittests')
        self.assertTrue(images)
        self.assertTrue(len(images) == 4)
        for i, j in enumerate(sorted(images)):
            self.assertTrue(j.endswith(os.path.join('canari', 'unittests', 'resources', 'images', '%d.png' % i)))
            self.assertTrue(os.path.lexists(j))

    def test_conf(self):
        self.assertTrue(global_config, 'Configuration file path could not be determined.')
        self.assertTrue(global_config.endswith('canari/resources/etc/canari.conf'))
        self.assertTrue(os.path.lexists(global_config))


from unittest import TestCase
from canari.config import *
from pkg_resources import resource_filename
from canari.mode import set_canari_mode, CanariMode
import os

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'CanariConfigParserTest'
]


class CanariConfigParserTest(TestCase):
    def setUp(self):
        self.root_config_file = resource_filename('canari.unittests.resources', 'root.conf')
        self.canari_test_resource_dir = resource_filename('canari.unittests.resources', '')
        self.config_parser = load_config(self.root_config_file)
        self.path_environ = os.environ['PATH']
        os.environ['CANARI_TEST_RESOURCE_DIR'] = self.canari_test_resource_dir

    def create_config(self, name, content):
        config_parser = CanariConfigParser()
        for section, d in content.iteritems():
            config_parser.add_section(section)
            for option, value in d.iteritems():
                config_parser.set(section, option, value)
        config_parser.write(file(name, mode='w'))

    def test_load_config(self):
        self.assertIsInstance(self.config_parser, CanariConfigParser)

    def test_no_recurse_load(self):
        self.assertSetEqual(set(self.config_parser.sections()),
                             {'default', 'canari.local', 'canari.remote', 'types', 'types.dotted'})

    def test_recurse_load(self):
        # TODO: find a way of locating the sub conf file in unittests
        # self.config_parser = load_config(self.root_config_file, True)
        # self.assertSetEqual(set(self.config_parser.sections()),
        #                      {'default', 'canari.local', 'canari.remote', 'types', 'types.dotted', 'subconf'})
        pass

    def test_get_option_from_sub_config(self):
        # TODO: find a way of locating the sub conf file in unittests
        # self.config_parser = load_config(self.root_config_file, True)
        # self.assertEqual('hello world', self.config_parser['subconf.test'])
        pass

    def test_get_int_option(self):
        self.assertEqual(1, self.config_parser['types.int'])

    def test_get_float_option(self):
        self.assertEqual(1.0, self.config_parser['types.float'])

    def test_get_str_option(self):
        self.assertEqual('hello world', self.config_parser['types.str'])

    def test_get_str_escaped_comma_option(self):
        self.assertEqual('1,2,3', self.config_parser['types.str_escaped_comma'])

    def test_get_str_env_var_option(self):
        self.assertEqual(self.path_environ, self.config_parser['types.str_env_var'])

    def test_get_str_env_var_inline_option(self):
        self.assertEqual('foo%sbar' % self.path_environ, self.config_parser['types.str_env_var_inline'])

    def test_get_str_quotes_option(self):
        self.assertEqual("'hello'", self.config_parser['types.str_quotes'])

    def test_get_str_double_quotes_option(self):
        self.assertEqual('"hello"', self.config_parser['types.str_double_quotes'])

    def test_get_str_no_var_exists_option(self):
        self.assertEqual('$foo', self.config_parser['types.str_no_var_exists'])

    def test_get_callable_option(self):
        self.assertEqual('object://os/system', self.config_parser['types.callable'])

    def test_get_list_option(self):
        self.assertListEqual(self.config_parser['types.list'], [1, 2, 3])

    def test_get_list_with_escaped_commas_option(self):
        self.assertListEqual(self.config_parser['types.list_with_escaped_commas'], [1, '2,3'])

    def test_get_list_with_mixed_types_option(self):
        self.assertListEqual(self.config_parser['types.list_with_mixed_types'],
                             [1, 1.0, 'foo', 'foo,bar', self.path_environ])

    def test_get_option_with_dot_delimeter(self):
        self.assertEqual('foo%sbar' % self.path_environ, self.config_parser['types.str_env_var_inline'])

    def test_get_option_with_more_than_one_dot(self):
        self.assertEqual("'hello'", self.config_parser['types.dotted.var'])

    def test_get_wordlist_option(self):
        self.assertListEqual(self.config_parser['types.wordlist'], ['1', '2', '3', '4', '5'])

    def test_get_from_invalid_section(self):
        self.assertRaises(NoSectionError, self.config_parser.__getitem__, 'foo.bar')

    def test_get_invalid_option_from_valid_section(self):
        self.assertRaises(NoOptionError, self.config_parser.__getitem__, 'default.foo')

    def test_add_section(self):
        self.config_parser += 'test'
        self.assertTrue('test' in self.config_parser.sections())
        self.assertTrue('test' in self.config_parser)
        self.config_parser.add_section('test2')
        self.assertTrue('test2' in self.config_parser.sections())
        self.assertTrue('test2' in self.config_parser)

    def test_add_section_and_option(self):
        self.config_parser['test.test'] = 1
        self.assertTrue('test' in self.config_parser and 'test.test' in self.config_parser)

    def set_option(self, value, expected_value=None, expected_backend_value=None):
        self.config_parser['test.test'] = value
        if isinstance(value, list):
            self.assertListEqual(expected_value or value, self.config_parser['test.test'])
        else:
            self.assertEqual(expected_value or value, self.config_parser['test.test'])
        self.assertEqual(expected_backend_value or expected_value or str(value),
                         self.config_parser.get('test', 'test'))

    def test_set_int_option(self):
        self.set_option(1)

    def test_set_float_option(self):
        self.set_option(0.5)

    def test_set_str_option(self):
        self.set_option('hello world')

    def test_set_str_env_var_option(self):
        self.set_option('${PATH}', self.path_environ)

    def test_set_str_env_var_innline_option(self):
        self.set_option('foo${PATH}bar', 'foo%sbar' % self.path_environ)

    def test_set_str_quotes_option(self):
        self.set_option("'hello'")

    def test_set_str_double_quotes_option(self):
        self.set_option('"hello"')

    def test_set_callable_option(self):
        def set_callable():
            self.set_option(os.system, expected_value='object://posix/system',
                            expected_backend_value='object://posix/system')
        self.assertRaises(ValueError, set_callable)

    def test_set_list_option(self):
        self.set_option([1, 2, 3], expected_backend_value='1,2,3')

    def test_set_list_with_escaped_comma_option(self):
        self.set_option([1, '2,3'], expected_backend_value='1,2\\,3')

    def test_set_list_with_mixed_types_option(self):
        self.set_option([1, 1.0, 'foo', 'foo,bar', '${PATH}'],
                        [1, 1.0, 'foo', 'foo,bar', self.path_environ],
                        '1,1.0,foo,foo\,bar,%s' % self.path_environ)

    def test_set_dotted_var_option(self):
        self.config_parser['test.test.test'] = 1
        self.assertEqual(1, self.config_parser['test.test.test'])
        self.assertEqual('1', self.config_parser.get('test.test', 'test'))

    def test_set_unknown_type_option(self):
        self.set_option({'a': 1}, str({'a': 1}))

    def test_write_config(self):
        with file('foo.conf', 'w') as f:
            self.config_parser.write(f)
        self.assertTrue(os.path.exists('foo.conf'))
        c = CanariConfigParser()
        c.read('foo.conf')
        self.assertListEqual(c.sections(), self.config_parser.sections())
        self.assertDictEqual(c._sections, self.config_parser._sections)
        os.unlink('foo.conf')

    def test_merge_config(self):
        c = CanariConfigParser()
        c['test.test'] = 1
        self.config_parser.update(c)
        self.assertTrue(self.config_parser.has_section('test'))
        self.assertTrue(self.config_parser.has_option('test', 'test'))
        self.assertEqual(1, self.config_parser['test.test'])

    def test_fail_merge_config(self):
        self.assertRaises(ValueError, self.config_parser.update, dict(d=1))

    def test_delete_option(self):
        del self.config_parser['default/paths']
        self.assertTrue('default' in self.config_parser)
        self.assertFalse('default/paths' in self.config_parser)

    def test_delete_section(self):
        del self.config_parser['default']
        self.assertFalse('default' in self.config_parser)

